/*********************************************************************
*                         COPYRIGHT NOTICE                           *
**********************************************************************
*        This software is copyright (C) 1982 by Pavel Curtis         *
*                                                                    *
*        Permission is granted to reproduce and distribute           *
*        this file by any means so long as no fee is charged         *
*        above a nominal handling fee and so long as this            *
*        notice is always included in the copies.                    *
*                                                                    *
*        Other rights are reserved except as explicitly granted      *
*        by written permission of the author.                        *
*                Pavel Curtis                                        *
*                Computer Science Dept.                              *
*                405 Upson Hall                                      *
*                Cornell University                                  *
*                Ithaca, NY 14853                                    *
*                                                                    *
*                Ph- (607) 256-4934                                  *
*                                                                    *
*                Pavel.Cornell@Udel-Relay   (ARPAnet)                *
*                decvax!cornell!pavel       (UUCPnet)                *
*********************************************************************/

/*
 * unctrl.h
 *
 * $Header:   RCS/unctrl.v  Revision 2.1  82/10/25  14:50:04  pavel  Exp$
 *
 * $Log:	RCS/unctrl.v $
Revision 2.1  82/10/25  14:50:04  pavel
Added Copyright Notice

Revision 2.0  82/10/24  15:18:28  pavel
Beta-one Test Release

Revision 1.3  82/08/23  22:31:24  pavel
The REAL Alpha-one Release Version

Revision 1.2  82/08/19  19:10:59  pavel
Alpha Test Release One

Revision 1.1  82/08/19  19:04:26  pavel
Initial revision

 */

extern char	*_unctrl[];

# define	unctrl(ch)	(_unctrl[(unsigned) ch])
