/*
 *	comp_captab.c -- The names of the capabilities in a form ready for
 *		         the making of a hash table for the compiler.
 *
 */


#include "compiler.h"
#include "term.h"


struct name_table_entry	cap_table[] =
{
	0,           "bw",	BOOLEAN,	  0,
	0,           "am",	BOOLEAN,	  1,
	0,          "xsb",	BOOLEAN,	  2,
	0,          "xhp",	BOOLEAN,	  3,
	0,         "xenl",	BOOLEAN,	  4,
	0,           "eo",	BOOLEAN,	  5,
	0,           "gn",	BOOLEAN,	  6,
	0,           "hc",	BOOLEAN,	  7,
	0,           "km",	BOOLEAN,	  8,
	0,           "hs",	BOOLEAN,	  9,
	0,           "in",	BOOLEAN,	 10,
	0,           "da",	BOOLEAN,	 11,
	0,           "db",	BOOLEAN,	 12,
	0,          "mir",	BOOLEAN,	 13,
	0,         "msgr",	BOOLEAN,	 14,
	0,           "os",	BOOLEAN,	 15,
	0,        "eslok",	BOOLEAN,	 16,
	0,           "xt",	BOOLEAN,	 17,
	0,           "hz",	BOOLEAN,	 18,
	0,           "ul",	BOOLEAN,	 19,
	0,          "xon",	BOOLEAN,	 20,
	0,         "cols",	NUMBER,		  0,
	0,           "it",	NUMBER,		  1,
	0,        "lines",	NUMBER,		  2,
	0,           "lm",	NUMBER,		  3,
	0,          "xmc",	NUMBER,		  4,
	0,           "pb",	NUMBER,		  5,
	0,           "vt",	NUMBER,		  6,
	0,          "wsl",	NUMBER,		  7,
	0,         "nlab",	NUMBER,		  8,
	0,           "lh",	NUMBER,		  9,
	0,           "lw",	NUMBER,		 10,
	0,          "cbt",	STRING,		  0,
	0,          "bel",	STRING,		  1,
	0,           "cr",	STRING,		  2,
	0,          "csr",	STRING,		  3,
	0,          "tbc",	STRING,		  4,
	0,        "clear",	STRING,		  5,
	0,           "el",	STRING,		  6,
	0,           "ed",	STRING,		  7,
	0,          "hpa",	STRING,		  8,
	0,           "CC",	STRING,		  9,
	0,          "cup",	STRING,		 10,
	0,         "cud1",	STRING,		 11,
	0,         "home",	STRING,		 12,
	0,        "civis",	STRING,		 13,
	0,         "cub1",	STRING,		 14,
	0,        "mrcup",	STRING,		 15,
	0,        "cnorm",	STRING,		 16,
	0,         "cuf1",	STRING,		 17,
	0,           "ll",	STRING,		 18,
	0,         "cuu1",	STRING,		 19,
	0,        "cvvis",	STRING,		 20,
	0,         "dch1",	STRING,		 21,
	0,          "dl1",	STRING,		 22,
	0,          "dsl",	STRING,		 23,
	0,           "hd",	STRING,		 24,
	0,        "smacs",	STRING,		 25,
	0,        "blink",	STRING,		 26,
	0,         "bold",	STRING,		 27,
	0,        "smcup",	STRING,		 28,
	0,         "smdc",	STRING,		 29,
	0,          "dim",	STRING,		 30,
	0,         "smir",	STRING,		 31,
	0,        "invis",	STRING,		 32,
	0,         "prot",	STRING,		 33,
	0,          "rev",	STRING,		 34,
	0,         "smso",	STRING,		 35,
	0,         "smul",	STRING,		 36,
	0,          "ech",	STRING,		 37,
	0,        "rmacs",	STRING,		 38,
	0,         "sgr0",	STRING,		 39,
	0,        "rmcup",	STRING,		 40,
	0,         "rmdc",	STRING,		 41,
	0,         "rmir",	STRING,		 42,
	0,         "rmso",	STRING,		 43,
	0,         "rmul",	STRING,		 44,
	0,        "flash",	STRING,		 45,
	0,           "ff",	STRING,		 46,
	0,          "fsl",	STRING,		 47,
	0,          "is1",	STRING,		 48,
	0,          "is2",	STRING,		 49,
	0,          "is3",	STRING,		 50,
	0,           "if",	STRING,		 51,
	0,         "ich1",	STRING,		 52,
	0,          "il1",	STRING,		 53,
	0,           "ip",	STRING,		 54,
	0,          "kbs",	STRING,		 55,
	0,         "ktbc",	STRING,		 56,
	0,         "kclr",	STRING,		 57,
	0,        "kctab",	STRING,		 58,
	0,        "kdch1",	STRING,		 59,
	0,         "kdl1",	STRING,		 60,
	0,        "kcud1",	STRING,		 61,
	0,        "krmir",	STRING,		 62,
	0,          "kel",	STRING,		 63,
	0,          "ked",	STRING,		 64,
	0,          "kf0",	STRING,		 65,
	0,          "kf1",	STRING,		 66,
	0,         "kf10",	STRING,		 67,
	0,          "kf2",	STRING,		 68,
	0,          "kf3",	STRING,		 69,
	0,          "kf4",	STRING,		 70,
	0,          "kf5",	STRING,		 71,
	0,          "kf6",	STRING,		 72,
	0,          "kf7",	STRING,		 73,
	0,          "kf8",	STRING,		 74,
	0,          "kf9",	STRING,		 75,
	0,        "khome",	STRING,		 76,
	0,        "kich1",	STRING,		 77,
	0,         "kil1",	STRING,		 78,
	0,        "kcub1",	STRING,		 79,
	0,          "kll",	STRING,		 80,
	0,          "knp",	STRING,		 81,
	0,          "kpp",	STRING,		 82,
	0,        "kcuf1",	STRING,		 83,
	0,         "kind",	STRING,		 84,
	0,          "kri",	STRING,		 85,
	0,         "khts",	STRING,		 86,
	0,        "kcuu1",	STRING,		 87,
	0,         "rmkx",	STRING,		 88,
	0,         "smkx",	STRING,		 89,
	0,          "lf0",	STRING,		 90,
	0,          "lf1",	STRING,		 91,
	0,         "lf10",	STRING,		 92,
	0,          "lf2",	STRING,		 93,
	0,          "lf3",	STRING,		 94,
	0,          "lf4",	STRING,		 95,
	0,          "lf5",	STRING,		 96,
	0,          "lf6",	STRING,		 97,
	0,          "lf7",	STRING,		 98,
	0,          "lf8",	STRING,		 99,
	0,          "lf9",	STRING,		100,
	0,          "rmm",	STRING,		101,
	0,          "smm",	STRING,		102,
	0,          "nel",	STRING,		103,
	0,          "pad",	STRING,		104,
	0,          "dch",	STRING,		105,
	0,           "dl",	STRING,		106,
	0,          "cud",	STRING,		107,
	0,          "ich",	STRING,		108,
	0,         "indn",	STRING,		109,
	0,           "il",	STRING,		110,
	0,          "cub",	STRING,		111,
	0,          "cuf",	STRING,		112,
	0,          "rin",	STRING,		113,
	0,          "cuu",	STRING,		114,
	0,        "pfkey",	STRING,		115,
	0,        "pfloc",	STRING,		116,
	0,          "pfx",	STRING,		117,
	0,          "mc0",	STRING,		118,
	0,          "mc4",	STRING,		119,
	0,          "mc5",	STRING,		120,
	0,          "rep",	STRING,		121,
	0,          "rs1",	STRING,		122,
	0,          "rs2",	STRING,		123,
	0,          "rs3",	STRING,		124,
	0,           "rf",	STRING,		125,
	0,           "rc",	STRING,		126,
	0,          "vpa",	STRING,		127,
	0,           "sc",	STRING,		128,
	0,          "ind",	STRING,		129,
	0,           "ri",	STRING,		130,
	0,          "sgr",	STRING,		131,
	0,          "hts",	STRING,		132,
	0,         "wind",	STRING,		133,
	0,           "ht",	STRING,		134,
	0,          "tsl",	STRING,		135,
	0,           "uc",	STRING,		136,
	0,           "hu",	STRING,		137,
	0,        "iprog",	STRING,		138,
	0,          "ka1",	STRING,		139,
	0,          "ka3",	STRING,		140,
	0,          "kb2",	STRING,		141,
	0,          "kc1",	STRING,		142,
	0,          "kc3",	STRING,		143,
	0,         "mc5p",	STRING,		144,
	0,          "rmp",	STRING,		145,
	0,         "acsc",	STRING,		146,
	0,          "pln",	STRING,		147,
};

struct name_table_entry *cap_hash_table[360];

int	Hashtabsize = 360;
int	Captabsize = 180;


#if (BOOLCOUNT!=21)||(NUMCOUNT!=11)||(STRCOUNT!=148)
	--> term.h and comp_captab.c disagree about the <--
	--> numbers of booleans, numbers and/or strings <--
#endif
